@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2001,2012,2020 LAAS/CNRS
+# Copyright (c) 2001,2012,2020,2023 LAAS/CNRS
 # All rights reserved.
 #
 #
@@ -282,12 +282,6 @@ namespace eval el {
 	# the matching rule (empty if none applies)
 	set rule ""
 
-	# just one weird case which is best worked around like this
-	if {[string index $string end] == ":" &&
-	    [string index $string end-1] != ":"} {
-	    append string ":"
-	}
-
 	# break the string up into its constituent elements
 	set parse [el::parse $string]
 	set end [lindex $parse 2]
@@ -305,14 +299,6 @@ namespace eval el {
 	    incr nitems
 	}
 
-        # get the namespace hierarchy
-        set namespaces [list]
-        catch {
-          set namespaces [namespace eval :: [subst {
-            namespace children {[namespace qualifiers [lindex $last 1]]}
-          }]]
-        }
-
 	# compute the kind of completion wanted, based on last item
 	set lasttokens [lindex $last 4]
 	set lasttoken [lindex $lasttokens end]
@@ -518,6 +504,9 @@ namespace eval el {
 	    set list [uplevel \#0 info vars [list [list ${name1}*]]]
 	    foreach match $list {
 		if { [uplevel \#0 array exists [list [list $match]]] } {
+                    if {[string match ::${name1}* $match]} {
+                      set match [string range $match 2 end]
+                    }
 		    lappend matches "$match { } ()"
 		}
 	    }
@@ -527,6 +516,9 @@ namespace eval el {
 	if { [lsearch $completeon "variable"] >= 0 } {
 	    set list [uplevel \#0 info vars [list [list ${name1}*]]]
 	    foreach match $list {
+                if {[string match ::${name1}* $match]} {
+                  set match [string range $match 2 end]
+                }
 		if { [uplevel \#0 array exists [list [list $match]]] } {
 		    lappend matches "$match ( ()"
 		} else {
@@ -540,13 +532,24 @@ namespace eval el {
 	    if { [lsearch $completeon $corp] >= 0 } {
 		set list [uplevel \#0 info $corp [list [list ${name1}*]]]
 		foreach match $list {
+                    if {[string match ::${name1}* $match]} {
+                      set match [string range $match 2 end]
+                    }
 		    lappend matches "$match { } {}"
 		}
 	    }
 	}
 
 	# namespaces
-	if { [lsearch $completeon "namespace"] >= 0 } {
+        if { [lsearch $completeon "namespace"] >= 0 } {
+            # get the namespace hierarchy
+            set namespaces [list]
+            catch {
+              set namespaces [namespace eval :: [subst {
+                namespace children {[namespace qualifiers $name1]}
+              }]]
+            }
+
 	    foreach namespace $namespaces {
 		if {[string match ${name1}* ${namespace}]} {
 		    lappend matches "${namespace}:: {} {}"
@@ -662,6 +665,14 @@ namespace eval el {
 	    }
 	}
 
+        # just one weird case which is best worked around like this
+        if {[string index $name1 end] == ":" &&
+            [string index $name1 end-1] != ":"} {
+          if {[llength [matches "$name1:"]] > 2} {
+            lappend matches "$name1: {} {}"
+          }
+        }
+
 	# last thing: if the incomplete word has a namespace or directory
 	# pattern which is present in all the matches, it is not
 	# necessary to replace it (completion will just append characters
@@ -676,7 +687,7 @@ namespace eval el {
 		incr slash [string length $pattern]
 		foreach match $matches {
 		    set string [lindex $match 0]
-		    if { ! [string equal -length $slash $name1 $match]} {
+		    if { ! [string equal -length $slash $name1 $string]} {
 			set ok 0
 			break
 		    }
